/*
 * Proyecto: Xailer
 * Fichero: Line.prg
 * Descripcin: Control lnea
 * Autor: Ignacio Ortiz de Ziga
 * Fecha: 21/12/2021
 */

#include "Xailer.ch"

//------------------------------------------------------------------------------

CLASS XLine FROM TControl

PUBLISHED:
   PROPERTY nWidth    INIT 100
   PROPERTY nHeight   INIT 100
   PROPERTY nPenColor INIT clLightGray WRITE INLINE ::SetPen( Value, nil, nil ) ;
                                       EDITOR PE_Color
   PROPERTY nPenSize  INIT 1           WRITE INLINE ::SetPen( nil, Value, nil )
   PROPERTY nPenStyle INIT psSolid     VALUES psSolid, psDash, psDot, psDashDot,;
                                       psDashDotDot, psNull, psInsideFrame ;
                                       WRITE INLINE ::SetPen( nil, nil, Value )
   PROPERTY lVertical INIT .T.         WRITE INLINE ::flVertical := Value, ::Refresh()

PUBLIC:
   METHOD Create( oParent ) CONSTRUCTOR
   METHOD Free()
   METHOD SetPen( nColor, nSize, nStyle )

RESERVED:
   PROPERTY nBorderStyle INIT bvNONE
   PROPERTY lTransparent INIT .T.
   PROPERTY lMultiline
   PROPERTY nOpacity     INIT 0
   PROPERTY lAlignKeepSpace
   PROPERTY lAlignShiftOnRotate
   PROPERTY lPathStyle
   PROPERTY nOpacity
   PROPERTY lAutoSize    INIT .F.
   PROPERTY cText
   PROPERTY lParentFont
   PROPERTY oFont
   PROPERTY oCursor
   PROPERTY oBrush
   PROPERTY nClrText
   PROPERTY nClrPane
   PROPERTY cTooltip
   PROPERTY cBalloon
   PROPERTY cTooltipTitle
   PROPERTY nTooltipIcon
   PROPERTY lEnabled
   PROPERTY HelpId
   PROPERTY nAlignWeight
   PROPERTY nAlignWeightSibling
   PROPERTY nVAlignment

   EVENT OnLButtonDown( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnLButtonUp( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnDblClick( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnRButtonDown( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnRButtonUp( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnMouseMove( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnShow( oSender ) // --> Nil
   EVENT OnHide( oSender ) // --> Nil
   EVENT OnContextMenu( oSender, nPosX, nPosY ) // --> Nil
   EVENT OnBeginDrag( oSender, nPosX, nPosY ) // --> lResult
   EVENT OnEndDrag( oSender, oFrom, nPosX, nPosY ) // --> Nil
   EVENT OnDragOver( oSender, oFrom, nPosX, nPosY ) // --> lResult
   EVENT OnClick( oSender ) // --> Nil
   EVENT OnDispTooltip( oSender, @cText, @cTitle, @nIcon, nItem ) // --> Nil
   EVENT OnPaint( oSender, hDC, @cPaintStruct ) // --> Nil
   EVENT OnPainted( oSender, hDC, @cPaintStruct ) // --> Nil

   METHOD WMEraseBkgnd()   INLINE 1
   METHOD WMPaint()

PROTECTED:
   DATA hPen INIT 0

END CLASS

//------------------------------------------------------------------------------

METHOD Create( oParent ) CLASS XLine

   ::Super:Create( oParent )

   ::cText := ""
   ::SetPen()

RETURN Self

//--------------------------------------------------------------------------

METHOD Free() CLASS XLine

   IF !Empty( ::hPen )
      DeleteObject( ::hPen )
      ::hPen := 0
   ENDIF

RETURN ::Super:Free()

//------------------------------------------------------------------------------

METHOD SetPen( nColor, nSize, nStyle ) CLASS XLine

   UPDATE ::FnPenColor TO nColor ,;
          ::FnPenSize  TO nSize  ,;
          ::FnPenStyle TO nStyle

   IF !Empty( ::hPen )
      DeleteObject( ::hPen )
   ENDIF

   ::hPen := CreatePen( ::FnPenStyle, ::FnPenSize, ::FnPenColor )

   ::Refresh()

RETURN Self

//------------------------------------------------------------------------------

#pragma BEGINDUMP

#include "windows.h"
#include "xailer.h"
#include "constants.ch"

HB_FUNC_STATIC( XLINE_WMPAINT )
{
   PHB_ITEM Self = hb_stackSelfItem();
   HWND hWnd = GetHandleOf( Self );
   CONTROLPAINT cp;
   RECT rect;
   int nPos;

   if ( XA_ControlBeginPaint( Self, hWnd, &cp, (HDC) hb_parnl( 1 ) ) )
      {
      GetClientRect( hWnd, &rect );
      XA_DrawControlBackground( Self, cp.hDC, &rect );
      SelectObject( cp.hDC, (HPEN) XA_ObjGetNL( Self, "hPen" ) );

      if ( XA_ObjGetL( Self, "lVertical" ) )
         {
         nPos = rect.left + ( ( rect.right  - rect.left ) >> 1 );
         MoveToEx( cp.hDC, nPos, rect.top + 1, NULL );
         LineTo( cp.hDC, nPos, rect.bottom );
         }
      else
         {
         nPos = rect.top + ( ( rect.bottom  - rect.top ) >> 1 );
         MoveToEx( cp.hDC, rect.left, nPos, NULL );
         LineTo( cp.hDC, rect.right, nPos);
         }
      }
   XA_ControlEndPaint( Self, hWnd, &cp );

   hb_retnl( 0 );
}

#pragma ENDDUMP
