/*
 * Proyecto: Xailer
 * Fichero: Rectangle.prg
 * Descripcin: Control Rectngulo
 * Autor: Ignacio Ortiz de Ziga
 * Fecha: 14/02/2022
 */

#include "Xailer.ch"

//------------------------------------------------------------------------------

CLASS XRectangle FROM TWinControl

PUBLISHED:
   PROPERTY nWidth    INIT 100
   PROPERTY nHeight   INIT 100
   PROPERTY nPenColor INIT clLightGray EDITOR PE_Color ;
                                       WRITE INLINE ::FnPenColor     := Value, ::Refresh()
   PROPERTY nPenSize       INIT 1      WRITE INLINE ::FnPenSize      := Value, ::Refresh()
   PROPERTY nOpacity       INIT 0      WRITE INLINE ::FnOpacity      := Value, ::Refresh()
   PROPERTY nBorderRadius  INIT 0      WRITE INLINE ::FnBorderRadius := Value, ::Refresh()
   PROPERTY lTransparent   INIT .F.    WRITE INLINE ::FlTransparent  := Value, ::Refresh()
   PROPERTY lFill          INIT .F.    WRITE INLINE ::FlFill         := Value, ::Refresh()

RESERVED:
   PROPERTY nBorderStyle INIT bvNONE
   PROPERTY lBorder      INIT .F.
   PROPERTY cText
   PROPERTY lParentFont
   PROPERTY oFont
   PROPERTY oCursor
   PROPERTY oBrush
   PROPERTY nClrText
//   PROPERTY nClrPane
   PROPERTY cTooltip
   PROPERTY cBalloon
   PROPERTY cTooltipTitle
   PROPERTY nTooltipIcon
   PROPERTY lEnabled
   PROPERTY HelpId
   PROPERTY nAlignWeight
   PROPERTY nAlignWeightSibling
   PROPERTY nVAlignment

   EVENT OnLButtonDown( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnLButtonUp( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnDblClick( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnRButtonDown( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnRButtonUp( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnMouseMove( oSender, nKeyFlags, nPosX, nPosY ) // --> Nil
   EVENT OnShow( oSender ) // --> Nil
   EVENT OnHide( oSender ) // --> Nil
   EVENT OnContextMenu( oSender, nPosX, nPosY ) // --> Nil
   EVENT OnBeginDrag( oSender, nPosX, nPosY ) // --> lResult
   EVENT OnEndDrag( oSender, oFrom, nPosX, nPosY ) // --> Nil
   EVENT OnDragOver( oSender, oFrom, nPosX, nPosY ) // --> lResult
   EVENT OnClick( oSender ) // --> Nil
   EVENT OnDispTooltip( oSender, @cText, @cTitle, @nIcon, nItem ) // --> Nil
   EVENT OnPaint( oSender, hDC, @cPaintStruct ) // --> Nil
   EVENT OnPainted( oSender, hDC, @cPaintStruct ) // --> Nil

   METHOD WMEraseBkgnd()   INLINE 1
   METHOD WMPaint()

END CLASS

//------------------------------------------------------------------------------

#pragma BEGINDUMP

#define WINVER 0x0500

#include <windows.h>
#include <xailer.h>
#include <Graphics.h>
#include <Gdip.h>

//------------------------------------------------------------------------------

HB_FUNC_STATIC( XRECTANGLE_WMPAINT )
{
   PHB_ITEM Self = hb_stackSelfItem();
   HWND hWnd = GetHandleOf( Self );
   CONTROLPAINT cp;

   if ( XA_ControlBeginPaint( Self, hWnd, &cp, (HDC) hb_parnl( 1 ) ) )
      {
      GpGraphics *g;
      GpPath *gPath;
      RECT rect;
      ARGB aColor;
      int nRadius   = XA_ObjGetNL( Self, "nBorderRadius" );
      int nPenSize  = XA_ObjGetNL( Self, "nPenSize" );
      int nPenColor = XA_ObjGetNL( Self, "nPenColor" );

      if ( !XA_ObjGetL( Self, "lTransparent" ) )
         aColor = XA_clr2argb( 255, nPenColor );
      else
         aColor = XA_clr2argb( MulDiv( XA_ObjGetNL( Self, "nOpacity" ), 255, 100 ),  nPenColor );

      GetClientRect( hWnd, &rect );
      XA_DrawControlBackground( Self, cp.hDC, &rect );

      GdipCreateFromHDC( cp.hDC, &g );
      GdipSetSmoothingMode( g, SmoothingModeAntiAlias );
      GdipSetCompositingQuality( g, CompositingQualityHighQuality );

      if ( XA_ObjGetL( Self, "lFill" ) )
         {
         GpBrush *gBrush;
         gPath = CreateRoundRect( rect, nRadius );
         GdipCreateSolidFill( aColor, &gBrush );
         GdipFillPath( g, gBrush, gPath );
         GdipDeleteBrush( gBrush );
         }
      else
         {
         GpPen *gPen;
         int nReduce = ( nPenSize >> 1 );
         InflateRect( &rect, - nReduce, - nReduce );
         gPath = CreateRoundRect( rect, nRadius );
         GdipCreatePen1( aColor, nPenSize, UnitPixel, &gPen );
         GdipDrawPath( g, gPen, gPath );
         GdipDeletePen( gPen );
         }

      GdipDeletePath( gPath );
      GdipDeleteGraphics( g );

      }
   XA_ControlEndPaint( Self, hWnd, &cp );

   hb_retnl( 0 );
}

#pragma ENDDUMP

//------------------------------------------------------------------------------

